package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type DosbingimbinganLogHandler struct {
	bimbinganLogUsecase usecase.BimbinganLogUsecase
}

func NewDosbingBimginganLogHandler(cfg config.BimbinganLogTransportConfig) *DosbingimbinganLogHandler {
	bimbinganLogRepository := repository.NewBimbinganLogRepository(cfg.DBWrite, cfg.DBRead)
	bimbinganLogUsecase := usecase.NewBimbinganLogUseCase(bimbinganLogRepository)

	return &DosbingimbinganLogHandler{
		bimbinganLogUsecase: bimbinganLogUsecase,
	}

}

func (t *DosbingimbinganLogHandler) MountDosbing(group *echo.Group) {
	group.POST("/update-status-bimbingan-log", t.UpdateBimbinganLogStatus)
	group.GET("/info-topik-mahasiswa", t.GetInfoTopikMahasiswa)
}

func (t *DosbingimbinganLogHandler) UpdateBimbinganLogStatus(c echo.Context) error {
	idLog := c.QueryParam("id_log")

	var req entity.UpdateStatusRequest
	if err := c.Bind(&req); err != nil {
		// Handle errors that occur during parsing of the JSON body
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Invalid request body", nil))
	}

	err := t.bimbinganLogUsecase.UpdateBimbinganLogStatus(idLog, req.Status)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Bimbingan log status successfully updated", nil))
}

func (t *DosbingimbinganLogHandler) GetInfoTopikMahasiswa(c echo.Context) error {
	idMahasiswa := c.QueryParam("id_mahasiswa")

	infoTopik, err := t.bimbinganLogUsecase.GetInfoTopikMahasiswa(idMahasiswa)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Info Topik successfully retreived", infoTopik))
}
