package repository

import (
	"errors"
	"fmt"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/entity"
	"gorm.io/gorm"
)

type DashboardDosbingRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewDashboardDosbingRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *DashboardDosbingRepo {
	return &DashboardDosbingRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *DashboardDosbingRepo) GetMahasiswaByDosenId(idDosen string, offset, size int, search string) (output []entity.MahasiswaBimbingan, err error) {
	var result *gorm.DB

	var mahasiswaBimbingan []entity.MahasiswaBimbingan

	var logBimbingan models.BimbinganLogs

	result = repo.DBRead.Table("dosen_bimbingan").
		Select(`pengguna.id AS id_mahasiswa, 
				pengguna.nama AS nama, 
				pengguna.nim AS nim, 
				topik.judul AS judul_topik, 
				topik.id AS id_topik`).
		Joins("JOIN pendaftaran_ta ON dosen_bimbingan.pendaftaran_id = pendaftaran_ta.id").
		Joins("JOIN pengguna ON pendaftaran_ta.id_mahasiswa = pengguna.id").
		Joins("JOIN topik ON pendaftaran_ta.id_topik = topik.id").
		Where("dosen_bimbingan.dosen_id = ?", idDosen).
		Where(" (pengguna.nama ILIKE '%' || ? || '%' OR pengguna.nim ILIKE '%' || ? || '%') ", search, search).
		Limit(size).
		Offset(offset).
		Scan(&mahasiswaBimbingan)

	if result.Error != nil {
		return nil, result.Error
	}

	// Assign status
	for i := range mahasiswaBimbingan {
		result := repo.DBRead.Where("id_mahasiswa = ?", mahasiswaBimbingan[i].IdMahasiswa).Order("date desc").First(&logBimbingan)

		if result.Error != nil {
			mahasiswaBimbingan[i].Status = "TERKENDALA"
		} else {
			currentDate := time.Now()
			diff := currentDate.Sub(logBimbingan.Date).Hours() / 24
			if diff <= 30 {
				mahasiswaBimbingan[i].Status = "LANCAR"
			} else if diff > 30 && diff < 90 {
				mahasiswaBimbingan[i].Status = "BUTUH BIMBINGAN"
			} else if diff >= 90 {
				mahasiswaBimbingan[i].Status = "TERKENDALA"
			}
		}
	}

	return mahasiswaBimbingan, nil

}

func (repo *DashboardDosbingRepo) GetAmountMahasiswaByDosenId(idDosen string) (output []entity.JumlahMahasiswa, err error) {
	var result *gorm.DB

	var jumlahMahasiswa entity.JumlahMahasiswa

	var mahasiswaIF int64
	var mahasiswaSTI int64

	// Mahasiswa IF
	result = repo.DBRead.Table("dosen_bimbingan").
		Joins("JOIN pendaftaran_ta ON dosen_bimbingan.pendaftaran_id = pendaftaran_ta.id").
		Joins("JOIN pengguna ON pendaftaran_ta.id_mahasiswa = pengguna.id").
		Where("dosen_bimbingan.dosen_id = ?", idDosen).
		Where("pengguna.nim LIKE '135%' ").
		Count(&mahasiswaIF)
	if result.Error != nil {
		fmt.Println(result)
		mahasiswaIF = 0
	}
	jumlahMahasiswa.MahasiswaIF = int(mahasiswaIF)

	// Mahasiswa STI
	result = repo.DBRead.Table("dosen_bimbingan").
		Joins("JOIN pendaftaran_ta ON dosen_bimbingan.pendaftaran_id = pendaftaran_ta.id").
		Joins("JOIN pengguna ON pendaftaran_ta.id_mahasiswa = pengguna.id").
		Where("dosen_bimbingan.dosen_id = ?", idDosen).
		Where("pengguna.nim LIKE '182%' ").
		Count(&mahasiswaSTI)
	if result.Error != nil {
		fmt.Println(result)
		mahasiswaSTI = 0
	}
	jumlahMahasiswa.MahasiswaSTI = int(mahasiswaSTI)

	return []entity.JumlahMahasiswa{jumlahMahasiswa}, nil

}

func (repo *DashboardDosbingRepo) GetAmountStatusBimbinganMahasiswaByDosenId(idDosen string) (output []entity.StatusBimbinganMahasiswa, err error) {
	var result *gorm.DB

	var statusBimbinganMahasiswa entity.StatusBimbinganMahasiswa
	var mahasiswaBimbingan []entity.MahasiswaBimbingan

	var logBimbingan models.BimbinganLogs

	result = repo.DBRead.Table("dosen_bimbingan").
		Select(`pengguna.id AS id_mahasiswa`).
		Joins("JOIN pendaftaran_ta ON dosen_bimbingan.pendaftaran_id = pendaftaran_ta.id").
		Joins("JOIN pengguna ON pendaftaran_ta.id_mahasiswa = pengguna.id").
		Where("dosen_bimbingan.dosen_id = ?", idDosen).
		Scan(&mahasiswaBimbingan)

	if result.Error != nil {
		return nil, errors.New("tidak ada mahasiswa bimbingan")
	}

	for _, mhs := range mahasiswaBimbingan {

		result = repo.DBRead.Where("id_mahasiswa = ?", mhs.IdMahasiswa).Order("date desc").First(&logBimbingan)

		if result.Error != nil {
			statusBimbinganMahasiswa.Terkendala++
		} else {
			currentDate := time.Now()
			diff := currentDate.Sub(logBimbingan.Date).Hours() / 24
			if diff <= 30 {
				statusBimbinganMahasiswa.Lancar++
			} else if diff > 30 && diff < 90 {
				statusBimbinganMahasiswa.ButuhBimbingan++
			} else if diff >= 90 {
				statusBimbinganMahasiswa.Terkendala++
			}
		}
	}

	return []entity.StatusBimbinganMahasiswa{statusBimbinganMahasiswa}, nil
}
