package repository

import (
	"errors"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_mahasiswa/entity"
	"gorm.io/gorm"
)

type DashboardMahasiswaRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewDashboardMahasiswaRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *DashboardMahasiswaRepo {
	return &DashboardMahasiswaRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *DashboardMahasiswaRepo) GetDashboardMahasiswaById(IdMahasiswa string) (output []entity.DashboardMahasiswaRet, err error) {
	var result *gorm.DB

	var dashboardMahasiswa entity.DashboardMahasiswaRet
	var dataMahasiswa entity.DataMahasiswa
	var statusPendaftaran entity.StatusPendaftaranTA
	var dataTopik entity.DataTopik
	var dataDosen entity.DataDosenBimbingan
	var logBimbingan []entity.LogBimbingan
	var berkasLogBimbingan []entity.BerkasLogBimbingan
	var dataSeminar entity.SidangSeminar
	var dataSidang entity.SidangSeminar

	var pendaftaranTA models.PendaftaranTA
	var logBimbinganModel []models.BimbinganLogs

	// Get Mahasiswa
	result = repo.DBRead.Table("pengguna").
		Select(`pengguna.id AS id_mahasiswa, 
				pengguna.nama AS nama, 
				pengguna.email AS email,
				pengguna.nim AS nim`).
		Where("pengguna.id = ?", IdMahasiswa).
		Scan(&dataMahasiswa)

	if result.Error != nil {
		return nil, errors.New("mahasiswa tidak ditemukan")
	}
	dashboardMahasiswa.Mahasiswa = dataMahasiswa

	result = repo.DBRead.Where("id_mahasiswa = ?", dashboardMahasiswa.Mahasiswa.IdMahasiswa).
		Order("waktu_pengiriman DESC").First(&pendaftaranTA)

	statusPendaftaran.Id = pendaftaranTA.ID
	statusPendaftaran.JalurPilihan = pendaftaranTA.JalurPilihan
	statusPendaftaran.WaktuPengiriman = pendaftaranTA.WaktuPengiriman
	statusPendaftaran.JadwalInterview = pendaftaranTA.InterviewAt
	statusPendaftaran.WaktuKeputusan = pendaftaranTA.DecidedAt
	statusPendaftaran.Status = string(pendaftaranTA.Status)

	// Cari data sesudah ini only if udah daftar
	if result.Error == nil {
		// Cari topik
		repo.DBRead.Table("pendaftaran_ta").
			Select(`topik.id,
					topik.judul,
					topik.deskripsi,
					topik.pengaju_id AS id_pengaju`).
			Joins("JOIN topik ON pendaftaran_ta.id_topik = topik.id").
			Where("pendaftaran_ta.id = ?", statusPendaftaran.Id).
			Scan(&dataTopik)

		statusPendaftaran.Topik = dataTopik
		dashboardMahasiswa.StatusPendaftaranTA = statusPendaftaran

		// Cari dosen
		result = repo.DBRead.Table("pendaftaran_ta").
			Select(`dosen_bimbingan.id AS id_dosen,
					pengguna.nama as nama,
					pengguna.email as email`).
			Joins("JOIN dosen_bimbingan ON pendaftaran_ta.id = dosen_bimbingan.pendaftaran_id").
			Joins("JOIN pengguna ON dosen_bimbingan.dosen_id = pengguna.id").
			Where("pendaftaran_ta.id = ?", statusPendaftaran.Id).
			Scan(&dataDosen)

		if result.Error == nil {

			dashboardMahasiswa.DosenBimbingan = dataDosen

			// Cari log bimbingan
			result = repo.DBRead.Where("id_mahasiswa = ?", dashboardMahasiswa.Mahasiswa.IdMahasiswa).
				Find(&logBimbinganModel)

			for i := range logBimbinganModel {
				repo.DBRead.Table("berkas").
					Select("id, nama, link").
					Where("id_bimbingan_log = ?", logBimbinganModel[i].ID).
					Scan(&berkasLogBimbingan)

				logBimbinganBaru := entity.LogBimbingan{
					Id:              logBimbinganModel[i].ID,
					WaktuBimbingan:  logBimbinganModel[i].Date,
					LaporanKemajuan: logBimbinganModel[i].LaporanKemajuan,
					Todo:            logBimbinganModel[i].Todo,
					NextBimbingan:   logBimbinganModel[i].NextBimbingan,
					Status:          logBimbinganModel[i].Status,
					Berkas:          berkasLogBimbingan,
				}

				logBimbingan = append(logBimbingan, logBimbinganBaru)
			}

			dashboardMahasiswa.LogBimbingan = logBimbingan

			// Cari seminar & sidang
			repo.DBRead.Table("pendaftaran_sidsem").
				Select(`pendaftaran_sidsem.id AS id, 
					pendaftaran_sidsem.ditolak AS ditolak, 
					pendaftaran_sidsem.lulus AS lulus, 
					pendaftaran_sidsem.waktu_mulai AS jadwal_seminar, 
					ruangan.nama AS ruangan`).
				Joins("JOIN ruangan ON pendaftaran_sidsem.ruangan_id = ruangan.id").
				Joins("JOIN pendaftaran_ta ON pendaftaran_sidsem.pendaftaran_ta_id = pendaftaran_ta.id").
				Where("pendaftaran_ta.id = ? AND pendaftaran_sidsem.tipe = ?", dashboardMahasiswa.StatusPendaftaranTA.Id, "SEMINAR").
				Scan(&dataSeminar)

			repo.DBRead.Table("pendaftaran_sidsem").
				Select(`pendaftaran_sidsem.id AS id, 
					pendaftaran_sidsem.ditolak AS ditolak, 
					pendaftaran_sidsem.lulus AS lulus, 
					pendaftaran_sidsem.waktu_mulai AS jadwal_seminar, 
					ruangan.nama AS ruangan`).
				Joins("JOIN ruangan ON pendaftaran_sidsem.ruangan_id = ruangan.id").
				Joins("JOIN pendaftaran_ta ON pendaftaran_sidsem.pendaftaran_ta_id = pendaftaran_ta.id").
				Where("pendaftaran_ta.id = ? AND pendaftaran_sidsem.tipe = ?", dashboardMahasiswa.StatusPendaftaranTA.Id, "SIDANG").
				Scan(&dataSidang)

			dashboardMahasiswa.Seminar = dataSeminar
			dashboardMahasiswa.Sidang = dataSidang
		}
	}

	return []entity.DashboardMahasiswaRet{dashboardMahasiswa}, nil
}
