package repository

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_tim_ta/entity"
	"gorm.io/gorm"
)

type DashboardTimRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewDashboardTimRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *DashboardTimRepo {
	return &DashboardTimRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *DashboardTimRepo) GetDataDashboard(offset, size int) (output []entity.DataPendaftaran, err error) {
	var dataMahasiswa []entity.DataPendaftaran

	var dosen models.Pengguna
	var mahasiswa models.Pengguna
	var pendaftaran []models.PendaftaranTA
	var logBimbingan []models.BimbinganLogs

	result := repo.DBRead.Limit(size).Offset(offset).Find(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	for _, pendaftar := range pendaftaran {
		res := repo.DBRead.Where("id = ?", pendaftar.IdMahasiswa).Limit(1).Find(&mahasiswa)
		if res.Error != nil {
			return nil, res.Error
		}
		res = repo.DBRead.Where("id = ?", pendaftar.IdDosen).Limit(1).Find(&dosen)
		if res.Error != nil {
			return nil, res.Error
		}

		data := entity.DataPendaftaran{
			IdPendaftaran:   pendaftar.ID,
			Nama:            mahasiswa.Nama,
			NIM:             mahasiswa.NIM,
			DosenPembimbing: dosen.Nama,
			Status:          "Pengajuan Topik",
		}

		if pendaftar.Status == "APPROVED" {
			data.Status = "Bimbingan"

			res := repo.DBRead.Where("id_mahasiswa = ?", pendaftar.IdMahasiswa).Find(&logBimbingan)
			if res.Error != nil {
				return nil, res.Error
			}
			if len(logBimbingan) >= 3 {
				data.Status = "Eligible Seminar Proposal"
				if len(logBimbingan) >= 6 {
					data.Status = "Eligible Sidang TA"
				}
			}
		}

		dataMahasiswa = append(dataMahasiswa, data)
	}

	return dataMahasiswa, nil

}

func (repo *DashboardTimRepo) GetStatusDashboard(offset, size int) (output []entity.StatusMahasiswa, err error) {
    var statusMahasiswa []entity.StatusMahasiswa
    var mahasiswa []models.Pengguna

    // Find all users that have role S1_MAHASISWA
    result := repo.DBRead.Where("roles @> ?", "{S1_MAHASISWA}").Limit(size).Offset(offset).Find(&mahasiswa)
    if result.Error != nil {
        return nil, result.Error
    }

    for _, mhs := range mahasiswa {
        var proposal models.PendaftaranTA
        var sidsem models.PendaftaranSidSem
        var taStatus, seminarStatus, sidangStatus bool

        // Check status for TA
        taResult := repo.DBRead.Where("id_mahasiswa = ?", mhs.ID).First(&proposal)
        if taResult.Error != nil {
            taStatus = false
        } else if proposal.Status != "APPROVED" {
            taStatus = false
        } else {
            taStatus = true
        }

        // Check status for Seminar Proposal
        seminarResult := repo.DBRead.Where("pendaftaran_ta_id = ? AND tipe = ?", proposal.ID, "Seminar Proposal").First(&sidsem)
        if seminarResult.Error != nil {
            seminarStatus = false
        } else if !sidsem.Ditolak {
            seminarStatus = false
        } else if sidsem.Ditolak {
            seminarStatus = true
        } else {
            seminarStatus = false
        }

        // Check status for Sidang Proposal
        sidangResult := repo.DBRead.Where("pendaftaran_ta_id = ? AND tipe = ?", mhs.ID, "Sidang Proposal").First(&sidsem)
        if sidangResult.Error != nil {
            sidangStatus = false
        } else if !sidsem.Ditolak {
            sidangStatus = false
        } else if sidsem.Ditolak {
            sidangStatus = true
        } else {
            sidangStatus = false
        }

        // Create StatusMahasiswa entity
        data := entity.StatusMahasiswa{
            Id:            mhs.ID,
            NIM:           mhs.NIM,
            Nama:          mhs.Nama,
            PengajuanTopik:      taStatus,
            SeminarProposal: seminarStatus,
            Sidang:  sidangStatus,
        }

        // Append to output
        statusMahasiswa = append(statusMahasiswa, data)
    }

    return statusMahasiswa, nil
}
