package usecase

import (
	"errors"
	"strings"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
)

type PendaftaranUsecase interface {
	AddPendaftaran(pendaftaran models.PendaftaranTA) (models.PendaftaranTA, error)
	GetPendaftaran() (output []models.PendaftaranTA, err error)
	DeletePendaftaran(id string) (output models.PendaftaranTA, err error)
	GetPendaftaranByIdDosen(id string) (output []models.PendaftaranTA, err error)
	UpdateStatusPendaftaran(id string, val string) (output []models.PendaftaranTA, err error)
	UpdateWaktuInterview(id string, val time.Time) (output []models.PendaftaranTA, err error)
	GetPendaftaranByPeriode(id string) (output []models.PendaftaranTA, err error)
	GetPendaftaranById(id string) (output []entity.PendaftaranGetResponse, err error)
	GetRekapPendaftaranDosen(id string) ([]entity.RekapPendaftaranRes, error)
	GetDetailRekapPendafataran(idDosen string, idMahasiswa string) (entity.DetailRekapPendaftara, error)
	UpdateInterview(interviewAt string, idDosen string, idMahasiswa string) error
	UpdateStatus(status string, idDosen string, idMahasiswa string) error
	GetStatisticsPendaftaran(idDosen string) (entity.Stats, error)
	GetListofDosbing() ([]entity.Pengguna, error)
	GetRekapPendaftaranTimta() ([]entity.RekapPendaftaranTimta, error)
	GetStatisticsPendaftaranTimta() (entity.Stats, error)
	UpdateStatusById(status string, idPendaftaran string) error
	GetRiwayatPendaftaran(idMahasiswa string) ([]entity.RiwayatPendaftaranRes, error)
	GetAllDosbing() ([]entity.DosenPembimbings, error)
	UpdateDosbing(idDosen []string, idPendaftaran string) error
}

type PendaftaranUc struct {
	pendaftaranRepo PendaftaranRepository
}

func NewPendaftaranUseCase(pendaftaranRepo PendaftaranRepository) *PendaftaranUc {
	return &PendaftaranUc{
		pendaftaranRepo: pendaftaranRepo,
	}
}

func (uc *PendaftaranUc) AddPendaftaran(pendaftaran models.PendaftaranTA) (models.PendaftaranTA, error) {
	newPendaftaran, err := uc.pendaftaranRepo.NewPendaftaran(pendaftaran)
	if err != nil {
		return newPendaftaran, err
	}

	return newPendaftaran, nil
}

func (uc *PendaftaranUc) GetPendaftaran() (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.GetPendaftaran()
}

func (uc *PendaftaranUc) DeletePendaftaran(id string) (output models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.DeletePendaftaran(id)
}

func (uc *PendaftaranUc) GetPendaftaranByIdDosen(id string) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.GetPendaftaranByIdDosen(id)
}

func (uc *PendaftaranUc) UpdateStatusPendaftaran(id string, val string) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.SetStatus(id, val)
}

func (uc *PendaftaranUc) UpdateWaktuInterview(id string, val time.Time) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.SetTimeInterview(id, val)
}

func (uc *PendaftaranUc) GetPendaftaranByPeriode(id string) (output []models.PendaftaranTA, err error) {
	return uc.pendaftaranRepo.GetPendaftaranByPeriode(id)
}

func (uc *PendaftaranUc) GetPendaftaranById(id string) (output []entity.PendaftaranGetResponse, err error) {
	return uc.pendaftaranRepo.GetPendaftaranById(id)
}

func (uc *PendaftaranUc) GetRekapPendaftaranDosen(id string) (output []entity.RekapPendaftaranRes, err error) {

	rekap, err := uc.pendaftaranRepo.GetRekapPendaftaranDosen(id)
	if err != nil {
		return []entity.RekapPendaftaranRes{}, err
	}
	for _, rek := range rekap {
		var jadwal string
		if rek.InterviewAt.IsZero() {
			jadwal = ""
		} else {
			jadwal = rek.InterviewAt.String()
			jadwal = strings.Replace(jadwal, " WIB", "", -1)
		}
		res := entity.RekapPendaftaranRes{
			ID:          rek.ID,
			IDMahasiswa: rek.IDMahasiswa,
			Nama:        rek.Nama,
			Nim:         rek.Nim,
			Status:      rek.Status,
			InterviewAt: jadwal,
		}
		output = append(output, res)
	}

	return output, nil
}

func (uc *PendaftaranUc) GetDetailRekapPendafataran(idDosen string, idMahasiswa string) (entity.DetailRekapPendaftara, error) {
	return uc.pendaftaranRepo.GetDetailRekapPendafataran(idDosen, idMahasiswa)
}

func (uc *PendaftaranUc) GetAllDosbing() ([]entity.DosenPembimbings, error) {
	return uc.pendaftaranRepo.GetAllDosbing()
}

func (uc *PendaftaranUc) UpdateInterview(interviewAt string, idDosen string, idMahasiswa string) error {
	if interviewAt == "" {
		return errors.New("interview timestamp cannot be empty")
	}
	return uc.pendaftaranRepo.UpdateInterview(interviewAt, idDosen, idMahasiswa)
}

func (uc *PendaftaranUc) UpdateStatus(status string, idDosen string, idMahasiswa string) error {
	if status == "" {
		return errors.New("interview timestamp cannot be empty")
	}
	return uc.pendaftaranRepo.UpdateStatus(status, idDosen, idMahasiswa)
}

func (uc *PendaftaranUc) UpdateDosbing(idDosen []string, idPendaftaran string) error {
	return uc.pendaftaranRepo.UpdateDosbing(idDosen[0], idPendaftaran)
}

func (uc *PendaftaranUc) UpdateStatusById(status string, idPendaftaran string) error {
	if status == "" {
		return errors.New("interview timestamp cannot be empty")
	}
	return uc.pendaftaranRepo.UpdateStatusById(status, idPendaftaran)
}

func (uc *PendaftaranUc) GetRiwayatPendaftaran(idMahasiswa string) ([]entity.RiwayatPendaftaranRes, error) {
	// Retrieve dosen pembimbing information
	dosBing, err := uc.pendaftaranRepo.GetDosbingForPendaftaran(idMahasiswa)
	if err != nil {
		return []entity.RiwayatPendaftaranRes{}, err
	}

	// Retrieve riwayat pendaftaran
	riwPen, err := uc.pendaftaranRepo.GetRiwayatPendaftaran(idMahasiswa)
	if err != nil {
		return []entity.RiwayatPendaftaranRes{}, err
	}

	var riwayatPendaftarans []entity.RiwayatPendaftaranRes
	for _, riw := range riwPen {
		var dosenPembimbings []entity.DosenPembimbing

		for _, dos := range dosBing {
			if dos.IDPendaftaran == riw.ID {
				dosenPembimbing := entity.DosenPembimbing{
					ID:   dos.ID,
					Nama: dos.Nama,
				}
				dosenPembimbings = append(dosenPembimbings, dosenPembimbing)
			}
		}

		// Create the RiwayatPendaftaranRes struct
		riwayatPendaftaran := entity.RiwayatPendaftaranRes{
			ID:              riw.ID,
			InterviewAt:     riw.InterviewAt,
			JalurPilihan:    riw.JalurPilihan,
			WaktuPengiriman: riw.WaktuPengiriman,
			Status:          riw.Status,
			Judul:           riw.Judul,
			Deskripsi:       riw.Deskripsi,
			DosenPembimbing: dosenPembimbings,
		}

		riwayatPendaftarans = append(riwayatPendaftarans, riwayatPendaftaran)
	}

	return riwayatPendaftarans, nil
}

func (uc *PendaftaranUc) GetRekapPendaftaranTimta() ([]entity.RekapPendaftaranTimta, error) {
	var rekapPendaftarans []entity.RekapPendaftaranTimta
	allPendaftaran, err := uc.pendaftaranRepo.GetAllPendaftaran()
	if err != nil {
		return []entity.RekapPendaftaranTimta{}, nil
	}
	for _, pendaftaran := range allPendaftaran {
		dosbingName, err := uc.pendaftaranRepo.GetDosbingName(pendaftaran.IDMahasiswa)
		if err != nil {
			return []entity.RekapPendaftaranTimta{}, nil
		}
		rekapPendaftaran := entity.RekapPendaftaranTimta{
			ID:             pendaftaran.IDMahasiswa,
			Nama:           pendaftaran.Nama,
			Nim:            pendaftaran.Nim,
			PembimbingNama: dosbingName,
			Status:         pendaftaran.Status,
			PendaftaranId:  pendaftaran.ID,
		}
		rekapPendaftarans = append(rekapPendaftarans, rekapPendaftaran)
	}
	return rekapPendaftarans, nil

}

func (uc *PendaftaranUc) GetStatisticsPendaftaran(idDosen string) (entity.Stats, error) {
	approved, err := uc.pendaftaranRepo.CountApproved(idDosen)
	if err != nil {
		return entity.Stats{}, err
	}
	proses, err := uc.pendaftaranRepo.CountProses(idDosen)
	if err != nil {
		return entity.Stats{}, err
	}
	rejected, err := uc.pendaftaranRepo.CountRejected(idDosen)
	if err != nil {
		return entity.Stats{}, err
	}
	total := approved + proses + rejected
	percApproved := approved / total * 100
	percProses := proses / total * 100
	perRejected := rejected / total * 100
	diterima := entity.StatsItem{
		Amount:     approved,
		Percentage: percApproved,
	}
	sedangProses := entity.StatsItem{
		Amount:     proses,
		Percentage: percProses,
	}
	ditolak := entity.StatsItem{
		Amount:     rejected,
		Percentage: perRejected,
	}
	stats := entity.Stats{
		Diterima:     diterima,
		SedangProses: sedangProses,
		Ditolak:      ditolak,
	}
	return stats, nil
}

func (uc *PendaftaranUc) GetListofDosbing() ([]entity.Pengguna, error) {
	dosbings, err := uc.pendaftaranRepo.GetListofDosbing()
	if err != nil {
		return nil, err
	}

	return dosbings, nil
}

func (uc *PendaftaranUc) GetStatisticsPendaftaranTimta() (entity.Stats, error) {
	approved, err := uc.pendaftaranRepo.CountApprovedAll()
	if err != nil {
		return entity.Stats{}, err
	}
	proses, err := uc.pendaftaranRepo.CountProsesAll()
	if err != nil {
		return entity.Stats{}, err
	}
	rejected, err := uc.pendaftaranRepo.CountRejectedAll()
	if err != nil {
		return entity.Stats{}, err
	}
	total := approved + proses + rejected
	percApproved := approved / total * 100
	percProses := proses / total * 100
	perRejected := rejected / total * 100
	diterima := entity.StatsItem{
		Amount:     approved,
		Percentage: percApproved,
	}
	sedangProses := entity.StatsItem{
		Amount:     proses,
		Percentage: percProses,
	}
	ditolak := entity.StatsItem{
		Amount:     rejected,
		Percentage: perRejected,
	}
	stats := entity.Stats{
		Diterima:     diterima,
		SedangProses: sedangProses,
		Ditolak:      ditolak,
	}
	return stats, nil
}
