package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/entity"
	"gorm.io/gorm"
)

type PeriodRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewPeriodRepo(dbWrite *gorm.DB, dbRead *gorm.DB) *PeriodRepo {
	return &PeriodRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *PeriodRepo) NewPeriodPendaftaran(period entity.PeriodPendaftaran) (output entity.PeriodPendaftaran, err error) {
	period.IDPeriod = uuid.New().String()

	period.Status = true

	err = repo.DBWrite.Model(&entity.PeriodPendaftaran{}).Where("status = ?", true).Update("status", false).Error
	if err != nil {
		return entity.PeriodPendaftaran{}, err
	}

	result := repo.DBWrite.Create(&period)
	if result.Error != nil {
		return entity.PeriodPendaftaran{}, result.Error
	}
	return period, nil
}

func (repo *PeriodRepo) GetPeriods() (output []entity.PeriodPendaftaran, err error) {
	var periods []entity.PeriodPendaftaran
	result := repo.DBRead.Find(&periods)
	if result.Error != nil {
		return nil, err
	}
	return periods, nil
}

func (repo *PeriodRepo) DeletePeriod(id string) (entity.PeriodPendaftaran, error) {
	period := entity.PeriodPendaftaran{IDPeriod: id}
	err := repo.DBWrite.Delete(&period).Error

	if err != nil {
		return period, err
	}

	return period, err
}

func (repo *PeriodRepo) GetPeriodCurrent() ([]entity.PeriodPendaftaran, error) {
	var periods []entity.PeriodPendaftaran
	currentTime := time.Now().Format("2006-01-02T15:04:05Z")
	result := repo.DBRead.Where(" ? BETWEEN start_date AND end_date", currentTime).Limit(1).Find(&periods)
	if result.Error != nil {
		return nil, result.Error
	}
	return periods, nil
}

func (repo *PeriodRepo) SetLivePeriode(id string) (entity.PeriodPendaftaran, error) {
	err := repo.DBWrite.Model(&entity.PeriodPendaftaran{}).Where("status = ?", true).Update("status", false).Error
	if err != nil {
		return entity.PeriodPendaftaran{}, err
	}

	period := entity.PeriodPendaftaran{IDPeriod: id}
	err = repo.DBWrite.Model(&period).Update("status", true).Error

	if err != nil {
		return period, err
	}

	return period, nil
}

func (repo *PeriodRepo) GetPeriodLive() ([]entity.PeriodPendaftaran, error) {
	var batch []entity.PeriodPendaftaran

	result := repo.DBRead.Where("status = ?", true).Limit(1).Find(&batch)
	if result.Error != nil {
		return nil, result.Error
	}

	return batch, nil
}
