package usecase

import (
	"strconv"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/period_pendaftaran/entity"
)

type PeriodPendaftaranUsecase interface {
	AddPeriod(period entity.PeriodPendaftaran) (entity.PeriodPendaftaran, error)
	GetPeriod() ([]entity.PeriodPendaftaran, error)
	DeletePeriod(id string) (entity.PeriodPendaftaran, error)
	GetPeriodCurrent() ([]entity.PeriodPendaftaran, error)
	SetLivePeriode(id string) (entity.PeriodPendaftaran, error)
	GetPeriodeLive() ([]entity.PeriodPendaftaran, error)
}

type PeriodPendaftaranUc struct {
	periodRepo PeriodPendaftaranRepository
}

func NewPeriodUseCase(periodRepo PeriodPendaftaranRepository) *PeriodPendaftaranUc {
	return &PeriodPendaftaranUc{
		periodRepo: periodRepo,
	}
}

func (uc *PeriodPendaftaranUc) AddPeriod(period entity.PeriodPendaftaran) (entity.PeriodPendaftaran, error) {
	startDate, _ := time.Parse("2006-01-02T15:04:05Z", period.StartDate)
	month := startDate.Month()
	year := startDate.Year()

	if month <= 6 {
		period.PeriodSemester = "II"
		period.PeriodYear = strconv.Itoa(year-1) + "/" + strconv.Itoa(year)
	} else {
		period.PeriodSemester = "I"
		period.PeriodYear = strconv.Itoa(year) + "/" + strconv.Itoa(year+1)
	}

	newPeriod, err := uc.periodRepo.NewPeriodPendaftaran(period)

	if err != nil {
		return newPeriod, err
	}

	return newPeriod, nil

}

func (uc *PeriodPendaftaranUc) GetPeriod() ([]entity.PeriodPendaftaran, error) {
	return uc.periodRepo.GetPeriods()
}

func (uc *PeriodPendaftaranUc) DeletePeriod(id string) (entity.PeriodPendaftaran, error) {
	return uc.periodRepo.DeletePeriod(id)
}

func (uc *PeriodPendaftaranUc) GetPeriodCurrent() ([]entity.PeriodPendaftaran, error) {
	return uc.periodRepo.GetPeriodCurrent()
}

func (uc *PeriodPendaftaranUc) SetLivePeriode(id string) (entity.PeriodPendaftaran, error) {
	return uc.periodRepo.SetLivePeriode(id)
}

func (uc *PeriodPendaftaranUc) GetPeriodeLive() ([]entity.PeriodPendaftaran, error) {
	return uc.periodRepo.GetPeriodLive()
}
