package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type TIMTASchedulingHandler struct {
	schedulingUsecase usecase.SchedulingUsecase
}

func NewTIMTASchedulingHandler(cfg config.SchedulingTransportConfig) *TIMTASchedulingHandler {
	schedulingRepository := repository.NewSchedulingRepository(cfg.DBWrite, cfg.DBRead)
	schedulingUsecase := usecase.NewSchedulingUseCase(schedulingRepository)

	return &TIMTASchedulingHandler{
		schedulingUsecase: schedulingUsecase,
	}

}

func (t *TIMTASchedulingHandler) MountTIMTA(group *echo.Group) {
	group.POST("/add-schedule-link", t.AddScheduleLink)
	group.GET("/scheduling", t.GetSchedulingTableData)
}

func (t *TIMTASchedulingHandler) AddScheduleLink(c echo.Context) error {
	idAndLink := new(entity.SchedulingLink)

	if err := c.Bind(idAndLink); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	err := t.schedulingUsecase.AddSchedulingLink(*idAndLink)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Link Successfully added", nil))
}

func (t *TIMTASchedulingHandler) GetSchedulingTableData(c echo.Context) error {
	tableData, err := t.schedulingUsecase.GetSchedulingTableData()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Link Successfully added", tableData))
}
